@@ -22,7 +22,7 @@ from __future__ import print_function
 
 __author__ = "Dilshod Temirkhodjaev <tdilshod@gmail.com>"
 __license__ = "GPL-2+"
-__version__ = "0.7.7"
+__version__ = "0.7.8"
 
 import csv, datetime, zipfile, string, sys, os, re, signal
 import xml.parsers.expat
@@ -179,6 +179,7 @@ class Xlsx2csv:
         options.setdefault("skip_empty_lines", False)
         options.setdefault("skip_trailing_columns", False)
         options.setdefault("escape_strings", False)
+        options.setdefault("no_line_breaks", False)
         options.setdefault("hyperlinks", False)
         options.setdefault("include_sheet_pattern", ["^.*$"])
         options.setdefault("exclude_sheet_pattern", [])
@@ -202,7 +203,9 @@ class Xlsx2csv:
         self.workbook = self._parse(Workbook, self.content_types.types["workbook"])
         workbook_relationships = list(filter(lambda r: "book" in r, self.content_types.types["relationships"]))[0]
         self.workbook.relationships = self._parse(Relationships, workbook_relationships)
-        if self.options['escape_strings']:
+        if self.options['no_line_breaks']:
+            self.shared_strings.replace_line_breaks()
+        elif self.options['escape_strings']:
             self.shared_strings.escape_strings()
 
     def __del__(self):
@@ -564,6 +567,10 @@ class SharedStrings:
         for i in range(0, len(self.strings)):
             self.strings[i] = self.strings[i].replace("\r", "\\r").replace("\n", "\\n").replace("\t", "\\t")
 
+    def replace_line_breaks(self):
+        for i in range(0, len(self.strings)):
+            self.strings[i] = self.strings[i].replace("\r", " ").replace("\n", " ").replace("\t", " ")
+
     def handleCharData(self, data):
         if self.t:
             self.value += data
@@ -767,20 +774,26 @@ class Sheet:
 
     def handleCharData(self, data):
         if self.in_cell_value:
+            format_type = None
+            format_str = "general"
             self.collected_string += data
             self.data = self.collected_string
             if self.colType == "s":  # shared string
+                format_type = "string"
                 self.data = self.sharedStrings[int(self.data)]
             elif self.colType == "b":  # boolean
+                format_type = "boolean"
                 self.data = (int(data) == 1 and "TRUE") or (int(data) == 0 and "FALSE") or data
             elif self.colType == "str" or self.colType == "inlineStr":
+                format_type = "string"
                 self.data = data
             elif self.s_attr:
                 s = int(self.s_attr)
 
                 # get cell format
-                format_str = "general"
-                xfs_numfmt = self.styles.cellXfs[s]
+                xfs_numfmt = None
+                if s < len(self.styles.cellXfs):
+                    xfs_numfmt = self.styles.cellXfs[s]
                 if xfs_numfmt in self.styles.numFmts:
                     format_str = self.styles.numFmts[xfs_numfmt]
                 elif xfs_numfmt in STANDARD_FORMATS:
@@ -791,7 +804,6 @@ class Sheet:
                     eprint("unknown format %s at %d" % (format_str, xfs_numfmt))
                     return
 
-                format_type = None
                 if format_str in FORMATS:
                     format_type = FORMATS[format_str]
                 elif re.match("^\d+(\.\d+)?$", self.data) and re.match(".*[hsmdyY]", format_str) and not re.match(
@@ -806,51 +818,54 @@ class Sheet:
                     format_type = "float"
                 if format_type == 'date' and self.dateformat == 'float':
                     format_type = "float"
-                if format_type and not format_type in self.ignore_formats:
-                    try:
-                        if format_type == 'date':  # date/time
-                            if self.workbook.date1904:
-                                date = datetime.datetime(1904, 1, 1) + datetime.timedelta(float(self.data))
-                            else:
-                                date = datetime.datetime(1899, 12, 30) + datetime.timedelta(float(self.data))
-                            if self.dateformat:
-                                # str(dateformat) - python2.5 bug, see: http://bugs.python.org/issue2782
-                                self.data = date.strftime(str(self.dateformat))
-                            else:
-                                # ignore ";@", don't know what does it mean right now
-                                # ignore "[$-409], [$-f409], [$-16001]" and similar format codes
-                                dateformat = re.sub(r"\[\$\-[A-z0-9]*\]", "", format_str, 1) \
-                                    .replace(";@", "").replace("yyyy", "%Y").replace("yy", "%y") \
-                                    .replace("hh:mm", "%H:%M").replace("h", "%I").replace("%H%H", "%H") \
-                                    .replace("ss", "%S").replace("dddd", "d").replace("dd", "d").replace("d", "%d") \
-                                    .replace("am/pm", "%p").replace("mmmm", "%B").replace("mmm", "%b") \
-                                    .replace(":mm", ":%M").replace("m", "%m").replace("%m%m", "%m")
-                                self.data = date.strftime(str(dateformat)).strip()
-                        elif format_type == 'time':  # time
-                            t = int(round((float(self.data) % 1) * 24 * 60 * 60, 6))  # it should be in seconds
-                            d = datetime.time(int((t // 3600) % 24), int((t // 60) % 60), int(t % 60))
-                            self.data = d.strftime(self.timeformat)
-                        elif format_type == 'float' and ('E' in self.data or 'e' in self.data):
-                            self.data = str(self.floatformat or '%f') % float(self.data)
-                        # if cell is general, be aggressive about stripping any trailing 0s, decimal points, etc.
-                        elif format_type == 'float' and format_str == 'general':
-                            self.data = ("%f" % (float(self.data))).rstrip('0').rstrip('.')
-                        elif format_type == 'float' and format_str[0:3] == '0.0':
-                            if self.floatformat:
-                                self.data = str(self.floatformat) % float(self.data)
-                            else:
-                                L = len(format_str.split(".")[1])
-                                if '%' in format_str:
-                                    L += 1
-                                self.data = ("%." + str(L) + "f") % float(self.data)
-                        elif format_type == 'float':
-                            # unsupported float formatting
-                            self.data = ("%f" % (float(self.data))).rstrip('0').rstrip('.')
+            elif self.colType == "n":
+                format_type = "float"
 
-                    except (ValueError, OverflowError):  # this catch must be removed, it's hiding potential problems
-                        eprint("Error: potential invalid date format.")
-                        # invalid date format
-                        pass
+            if format_type and not format_type in self.ignore_formats:
+                try:
+                    if format_type == 'date':  # date/time
+                        if self.workbook.date1904:
+                            date = datetime.datetime(1904, 1, 1) + datetime.timedelta(float(self.data))
+                        else:
+                            date = datetime.datetime(1899, 12, 30) + datetime.timedelta(float(self.data))
+                        if self.dateformat:
+                            # str(dateformat) - python2.5 bug, see: http://bugs.python.org/issue2782
+                            self.data = date.strftime(str(self.dateformat))
+                        else:
+                            # ignore ";@", don't know what does it mean right now
+                            # ignore "[$-409], [$-f409], [$-16001]" and similar format codes
+                            dateformat = re.sub(r"\[\$\-[A-z0-9]*\]", "", format_str, 1) \
+                                .replace(";@", "").replace("yyyy", "%Y").replace("yy", "%y") \
+                                .replace("hh:mm", "%H:%M").replace("h", "%I").replace("%H%H", "%H") \
+                                .replace("ss", "%S").replace("dddd", "d").replace("dd", "d").replace("d", "%d") \
+                                .replace("am/pm", "%p").replace("mmmm", "%B").replace("mmm", "%b") \
+                                .replace(":mm", ":%M").replace("m", "%m").replace("%m%m", "%m")
+                            self.data = date.strftime(str(dateformat)).strip()
+                    elif format_type == 'time':  # time
+                        t = int(round((float(self.data) % 1) * 24 * 60 * 60, 6))  # it should be in seconds
+                        d = datetime.time(int((t // 3600) % 24), int((t // 60) % 60), int(t % 60))
+                        self.data = d.strftime(self.timeformat)
+                    elif format_type == 'float' and ('E' in self.data or 'e' in self.data):
+                        self.data = str(self.floatformat or '%f') % float(self.data)
+                    # if cell is general, be aggressive about stripping any trailing 0s, decimal points, etc.
+                    elif format_type == 'float' and format_str == 'general':
+                        self.data = ("%f" % (float(self.data))).rstrip('0').rstrip('.')
+                    elif format_type == 'float' and format_str[0:3] == '0.0':
+                        if self.floatformat:
+                            self.data = str(self.floatformat) % float(self.data)
+                        else:
+                            L = len(format_str.split(".")[1])
+                            if '%' in format_str:
+                                L += 1
+                            self.data = ("%." + str(L) + "f") % float(self.data)
+                    elif format_type == 'float':
+                        # unsupported float formatting
+                        self.data = ("%f" % (float(self.data))).rstrip('0').rstrip('.')
+
+                except (ValueError, OverflowError):  # this catch must be removed, it's hiding potential problems
+                    eprint("Error: potential invalid date format.")
+                    # invalid date format
+                    pass
 
     def handleStartElement(self, name, attrs):
         has_namespace = name.find(":") > 0
@@ -920,12 +935,22 @@ class Sheet:
 
         if self.in_row and (name == 'row' or (has_namespace and name.endswith(':row'))):
             if len(self.columns.keys()) > 0:
-                d = [""] * (max(self.columns.keys()) + 1)
-                for k in self.columns.keys():
-                    val = self.columns[k]
-                    if not self.py3:
-                        val = val.encode("utf-8")
-                    d[k] = val
+                if min(self.columns.keys()) < 0: # Weird
+                    d = []
+                    keys = self.columns.keys()
+                    keys.sort()
+                    for k in keys:
+                        val = self.columns[k]
+                        if not self.py3:
+                            val = val.encode("utf-8")
+                        d.append(val)
+                else:
+                    d = [""] * (max(self.columns.keys()) + 1)
+                    for k in self.columns.keys():
+                        val = self.columns[k]
+                        if not self.py3:
+                            val = val.encode("utf-8")
+                        d[k] = val
                 if self.spans:
                     l = self.spans[1]
                     if len(d) < l:
@@ -1036,6 +1061,8 @@ if __name__ == "__main__":
                         help="include hyperlinks")
     parser.add_argument("-e", "--escape", dest='escape_strings', default=False, action="store_true",
                         help="Escape \\r\\n\\t characters")
+    parser.add_argument("--no-line-breaks", "--no-line-breaks", dest='no_line_breaks', default=False, action="store_true",
+                        help="Replace \\r\\n\\t with space")
     parser.add_argument("-E", "--exclude_sheet_pattern", nargs=nargs_plus, dest="exclude_sheet_pattern", default="",
                         help="exclude sheets named matching given pattern, only effects when -a option is enabled.")
     parser.add_argument("-f", "--dateformat", dest="dateformat",
@@ -1140,6 +1167,7 @@ if __name__ == "__main__":
         'skip_empty_lines': options.skip_empty_lines,
         'skip_trailing_columns': options.skip_trailing_columns,
         'escape_strings': options.escape_strings,
+        'no_line_breaks': options.no_line_breaks,
         'hyperlinks': options.hyperlinks,
         'include_sheet_pattern': options.include_sheet_pattern,
         'exclude_sheet_pattern': options.exclude_sheet_pattern,
